/*
 * Author: Dušan Kolář
 */

#include <iostream>
#include <fstream>

#include <stdio.h>
#include <malloc.h>

#include "iec104.h"


void demo_parse_packet(const char * data, long packet_size) {
    std::string cs(data,(size_t)packet_size);
    kaitai::kstream ks(cs);
    
    std::cout << "Created kstream ..." <<  std::endl;
    
    iec104_t parser(&ks);
    
    const char *magic_data = parser.magic().data();
    
    std::cout << "Magic 68 (" << 0x68 << ") = " << std::hex << (int)*magic_data << " (" << std::dec << (int)*magic_data << ") [" <<  parser.magic() << "]" << std::endl;

    std::cout << "Packet length: " << (int)parser.length() << std::endl;
    
    std::cout << "Is I-Form: " << std::boolalpha << (! parser.flag0()) << " ~ " << (! parser._is_null_iform()) << std::endl;
    std::cout << "Is S-Form: " << std::boolalpha << (parser.flag0() && (!parser.flag1())) << " ~ " << (! parser._is_null_sform()) << std::endl;
    std::cout << "Is U-Form: " << std::boolalpha << (parser.flag0() && parser.flag1()) << " ~ " << (! parser._is_null_uform()) << std::endl;
    
    if (!parser.flag0()) {
        // I-Form
        iec104_t::iform_t_t* iform_val = parser.iform();
        std::cout << "Zero bit: " << iform_val->zero_bit() << std::endl;
        std::cout << "Send sequence no: " << iform_val->send_no() << std::endl;
        std::cout << "Receive sequence no: " << iform_val->rec_no() << std::endl;
    } else
    if (parser.flag0() && !parser.flag1()) {
        // S-Form
        iec104_t::sform_t_t* sform_val = parser.sform();
        std::cout << "Zero bit: " << sform_val->zero_bit() << std::endl;
        std::cout << "Receive sequence no: " << sform_val->rec_no() << std::endl;
    } else {
        // U-Form
        iec104_t::uform_t_t* uform_val = parser.uform();
        std::cout << "Zero bit: " << uform_val->zero_bit() << std::endl;
        std::cout << "Testfr: " << parser.testfr() << std::endl;
        std::cout << "Stopdt: " << parser.stopdt() << std::endl;
        std::cout << "Startdt: " << parser.startdt() << std::endl;
    }
    
    std::cout << "Is ASDU? " << std::boolalpha << (! parser._is_null_asdu()) << " ~ " << (parser.length() > 10) << std::endl;
    
    if (parser.length()>10) {
        // ASDU
        iec104_t::asdu_t_t* asdu_val = parser.asdu();
        std::cout << "Type identification: " << (int)asdu_val->type_identification() << std::endl;
        std::cout << "Structure qualifier: " << asdu_val->sq() << std::endl;
        std::cout << "Number of items: " << asdu_val->num_of_items() << std::endl;
        std::cout << "Test? " << std::boolalpha << asdu_val->t() << std::endl;
        std::cout << "Positive (false)/negative (true) confirmation: " << std::boolalpha << asdu_val->p_n() << std::endl;
        std::cout << "Cause of transmission: " << asdu_val->cot() << std::endl;
        std::cout << "Originator address: " << (int)asdu_val->org() << std::endl;
        std::cout << "ASDU address: " << asdu_val->address() << std::endl;
    }
    
}

int main(int argc, char *argv[]) {
    if (argc<2) {
        std::cout << "Usage: test <iec104_packet_file(s)>" <<  std::endl;
        return 1;
    }
    
    for (int i=1; i<argc;++i) {
        std::cout << "Started " << argv[i] << "..." <<  std::endl;
    
        FILE *packet;
        if ((packet = fopen(argv[i],"rb")) == NULL) {
            std::cout << "Cannot open file: " << argv[i] <<  std::endl;
            return 2;
        }
        
        if (fseek(packet, 0L, SEEK_END)) {
            std::cout << "Cannot work with file: " << argv[i] <<  std::endl;
            fclose(packet);
            return 3;
        }
        
        long packet_size = ftell(packet);
        
        if (packet_size<0) {
            std::cout << "Cannot work with file: " << argv[i] <<  std::endl;
            fclose(packet);
            return 4;
        }
        
        if (packet_size > 257L) {
            std::cout << "Packet size too large: " << packet_size <<  std::endl;
            fclose(packet);
            return 5;
        }
        
        if (fseek(packet, 0L, SEEK_SET)) {
            std::cout << "Cannot work with file: " << argv[i] <<  std::endl;
            fclose(packet);
            return 6;
        }
        
        char *data =(char*)malloc((size_t)packet_size);
        
        if (fread((void*)data,1,(size_t)packet_size,packet) != (size_t)packet_size) {
            std::cout << "Cannot read file: " << argv[i] <<  std::endl;
            fclose(packet);
            return 7;
        }
        
        fclose(packet);
        
        demo_parse_packet(data, packet_size);
        
        free((void*)data);
        
        std::cout << "Done " << argv[i] << " !" << std::endl << std::endl;
        
    } // end for
    
    return 0;
} // main

// EOF
